@@ -201,13 +201,7 @@ calCachedCalendar.prototype = {
                     case "storage":
                         let file = getCalendarDirectory();
                         file.append("cache.sqlite");
-                        let uri = getIOService().newFileURI(file);
-                        // use same id as uncached calendar:
-                        let idParam = this.uri.spec.indexOf("?id=");
-                        if (idParam != -1) {
-                            uri.spec += this.uri.spec.substring(idParam);
-                        }
-                        cachedCalendar.uri = uri;
+                        cachedCalendar.uri = getIOService().newFileURI(file);
                         cachedCalendar.id = this.id;
                         break;
                     default:
@@ -261,10 +255,11 @@ calCachedCalendar.prototype = {
         if (this.supportsChangeLog) {
             LOG("[calCachedCalendar] Doing changelog based sync for calendar " + this.uri.spec);
             var opListener = {
+                thisCalendar : this,
                 onResult: function(op, result) {
                     if (!op || !op.isPending) {
                         var status = (op ? op.status : Components.results.NS_OK);
-                        ASSERT(Components.isSuccessCode(status), "replay action failed: " + (op ? op.id : "<unknown>"));
+                        ASSERT(Components.isSuccessCode(status), "replay action failed: " + (op ? op.id : "<unknown>")+", uri=" + this.thisCalendar.uri.spec + ", result="  +result + ", op=" + op);
                         LOG("[calCachedCalendar] replayChangesOn finished.");
                         emptyQueue(status);
                     }
@@ -494,4 +489,3 @@ calCachedCalendar.prototype = {
     defineForwards(calCachedCalendar.prototype, "mCachedCalendar",
                    ["getItem", "getItems", "startBatch", "endBatch"], [], []);
 })();
-
