@@ -647,7 +647,13 @@ NS_IMETHODIMP nsMailtoUrl::SchemeIs(const char *aScheme, PRBool *_retval)
 
 NS_IMETHODIMP nsMailtoUrl::Equals(nsIURI *other, PRBool *_retval)
 {
-	return m_baseURL->Equals(other, _retval);
+  // The passed-in URI might be an nsMailtoUrl. Pass our inner URL to its
+  // Equals method. The other nsMailtoUrl will then pass its inner URL to
+  // to the Equals method of our inner URL. Other URIs will return false.
+  if (other)
+    return other->Equals(m_baseURL, _retval);
+
+  return m_baseURL->Equals(other, _retval);
 }
 
 NS_IMETHODIMP nsMailtoUrl::Clone(nsIURI **_retval)
@@ -735,6 +741,20 @@ NS_IMETHODIMP nsSmtpUrl::GetPostMessageFile(nsIFile ** aFile)
 NS_IMPL_GETSET(nsSmtpUrl, RequestDSN, PRBool, m_requestDSN)
 
 NS_IMETHODIMP 
+nsSmtpUrl::SetDsnEnvid(const nsACString &aDsnEnvid)
+{
+    m_dsnEnvid = aDsnEnvid;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsSmtpUrl::GetDsnEnvid(nsACString &aDsnEnvid)
+{
+    aDsnEnvid = m_dsnEnvid;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
 nsSmtpUrl::GetSenderIdentity(nsIMsgIdentity * *aSenderIdentity)
 {
   NS_ENSURE_ARG_POINTER(aSenderIdentity); 
