@@ -298,7 +298,7 @@ mime_new (MimeObjectClass *clazz, MimeHeaders *hdrs,
   object->dontShowAsAttachment = PR_FALSE;
 
   if (override_content_type && *override_content_type)
-  object->content_type = strdup(override_content_type);
+    object->content_type = strdup(override_content_type);
 
   status = clazz->initialize(object);
   if (status < 0)
@@ -681,6 +681,15 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
                       !PL_strcasecmp(micalg, PARAM_MICALG_SHA1_3) ||
                       !PL_strcasecmp(micalg, PARAM_MICALG_SHA1_4) ||
                       !PL_strcasecmp(micalg, PARAM_MICALG_SHA1_5) ||
+                      !PL_strcasecmp(micalg, PARAM_MICALG_SHA256) ||
+                      !PL_strcasecmp(micalg, PARAM_MICALG_SHA256_2) ||
+                      !PL_strcasecmp(micalg, PARAM_MICALG_SHA256_3) ||
+                      !PL_strcasecmp(micalg, PARAM_MICALG_SHA384) ||
+                      !PL_strcasecmp(micalg, PARAM_MICALG_SHA384_2) ||
+                      !PL_strcasecmp(micalg, PARAM_MICALG_SHA384_3) ||
+                      !PL_strcasecmp(micalg, PARAM_MICALG_SHA512) ||
+                      !PL_strcasecmp(micalg, PARAM_MICALG_SHA512_2) ||
+                      !PL_strcasecmp(micalg, PARAM_MICALG_SHA512_3) ||
                       !PL_strcasecmp(micalg, PARAM_MICALG_MD2))))
             clazz = (MimeObjectClass *)&mimeMultipartSignedCMSClass;
           else
@@ -857,22 +866,18 @@ mime_create (const char *content_type, MimeHeaders *hdrs,
     if (name)
     {
       override_content_type = opts->file_type_fn (name, opts->stream_closure);
+      // appledouble isn't a valid override content type, and makes
+      // attachments invisible.
+      if (!PL_strcasecmp(override_content_type, MULTIPART_APPLEDOUBLE))
+        override_content_type = nsnull;
       PR_FREEIF(name);
 
-      // Of, if we got here and it is not the unknown content type from the
-      // file name, lets do some better checking not to inline something bad
-      //
-      if (override_content_type && (PL_strcasecmp(override_content_type, UNKNOWN_CONTENT_TYPE)))
-      {
-        // Only inline this if it makes sense to do so!
-        if ( (!content_type) ||
-             (content_type && (!PL_strcasecmp(content_type, UNKNOWN_CONTENT_TYPE))) )
-        {
-          content_type = override_content_type;
-        }
-        else
-          PR_FREEIF(override_content_type);
-      }
+      // If we get here and it is not the unknown content type from the
+      // file name, let's do some better checking not to inline something bad
+      if (override_content_type &&
+          *override_content_type &&
+          (PL_strcasecmp(override_content_type, UNKNOWN_CONTENT_TYPE)))
+        content_type = override_content_type;
     }
   }
 
