@@ -78,10 +78,11 @@ namespace storage {
 
 #define PREF_TS_SYNCHRONOUS "toolkit.storage.synchronous"
 
+namespace {
+
 ////////////////////////////////////////////////////////////////////////////////
-//// sqlite3_context Specialization Functions
+//// Variant Specialization Functions (variantToSQLiteT)
 
-template < >
 int
 sqlite3_T_int(sqlite3_context *aCtx,
               int aValue)
@@ -90,7 +91,6 @@ sqlite3_T_int(sqlite3_context *aCtx,
   return SQLITE_OK;
 }
 
-template < >
 int
 sqlite3_T_int64(sqlite3_context *aCtx,
                 sqlite3_int64 aValue)
@@ -99,7 +99,6 @@ sqlite3_T_int64(sqlite3_context *aCtx,
   return SQLITE_OK;
 }
 
-template < >
 int
 sqlite3_T_double(sqlite3_context *aCtx,
                  double aValue)
@@ -108,19 +107,28 @@ sqlite3_T_double(sqlite3_context *aCtx,
   return SQLITE_OK;
 }
 
-template < >
+int
+sqlite3_T_text(sqlite3_context *aCtx,
+               const nsCString &aValue)
+{
+  ::sqlite3_result_text(aCtx,
+                        aValue.get(),
+                        aValue.Length(),
+                        SQLITE_TRANSIENT);
+  return SQLITE_OK;
+}
+
 int
 sqlite3_T_text16(sqlite3_context *aCtx,
-                 nsString aValue)
+                 const nsString &aValue)
 {
   ::sqlite3_result_text16(aCtx,
-                          PromiseFlatString(aValue).get(),
+                          aValue.get(),
                           aValue.Length() * 2, // Number of bytes.
                           SQLITE_TRANSIENT);
   return SQLITE_OK;
 }
 
-template < >
 int
 sqlite3_T_null(sqlite3_context *aCtx)
 {
@@ -128,7 +136,6 @@ sqlite3_T_null(sqlite3_context *aCtx)
   return SQLITE_OK;
 }
 
-template < >
 int
 sqlite3_T_blob(sqlite3_context *aCtx,
                const void *aData,
@@ -138,10 +145,11 @@ sqlite3_T_blob(sqlite3_context *aCtx,
   return SQLITE_OK;
 }
 
+#include "variantToSQLiteT_impl.h"
+
 ////////////////////////////////////////////////////////////////////////////////
 //// Local Functions
 
-namespace {
 #ifdef PR_LOGGING
 void tracefunc (void *aClosure, const char *aStmt)
 {
@@ -484,7 +492,7 @@ Connection::Close()
   nsCAutoString leafName(":memory");
   if (mDatabaseFile)
       (void)mDatabaseFile->GetNativeLeafName(leafName);
-  PR_LOG(gStorageLog, PR_LOG_NOTICE, ("Closing connection to '%s'",
+  PR_LOG(gStorageLog, PR_LOG_NOTICE, ("Opening connection to '%s'",
                                       leafName.get()));
 #endif
 
@@ -649,7 +657,7 @@ Connection::ExecuteAsync(mozIStorageStatement **aStatements,
     nsresult rv = stmt->getAsynchronousStatementData(data);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    NS_ASSERTION(stmt->owningConnection() == this,
+    NS_ASSERTION(::sqlite3_db_handle(stmt->nativeStatement()) == mDBConn,
                  "Statement must be from this database connection!");
 
     // Now append it to our array.
